(document:surround "/std/frame")
(document:insert "/std/functions")

(document:insert "/vm/functions")

(document:envelop with-attributes (option set-value! has-value? got-value need-value apply-value))

(document:envelop with-container-presentations
                                    ((option-widget '/vm/option option)))

; /dev/evms/path/to/obj -> ("path" "to" "obj")
(define (remove-evms-prefix path)
  (string-split
    (if (string-prefix? "/dev/evms/" path)
      (substring path 10)
      path) #\/))

; /path/to/volume_luks -> /path/to/volume
(define (remove-luks-suffix path)
  (if (string-suffix? "_luks" path)
    (string-drop-right path 5)
    path))

(define (normalize-btrfs-path path)
  (cons (car path)
        (cond ((member (cadr path) '("imsm" "md"))
               ; btrfs/md/md0/subvol - remove md
               ; btrfs/imsm/imsm_Volume0_0_part3 - remove imsm
               (cddr path))
              ((equal? (cadr path) "lvm2")
               ; btrfs/lvm2/vg/vol/subvol - remove lvm2 and volume group
               (cdddr path))
              (#t (cdr path)))))

; /dev/sda1 -> /dev/sda
; /dev/nvmen1p1 -> /dev/nvmen1
; /dev/imsm/Volume0_0_part1 -> /dev/imsm/Volume0_0
; path is list
(define (remove-partition path)
  (cond
    ; nvme/mmcblk
    ((or (string-prefix? "nvme" (car path))
         (string-prefix? "mmcblk" (car path)))
     (let ((trimmed (string-trim-right (remove-luks-suffix (car path)) char-set:digit)))
       (cons "disk"
             (if (string-suffix? "p" trimmed)
               (cons (string-drop-right trimmed 1) path)
               path))))
    ((string-prefix? "mpath" (car path))
     (let ((trimmed (string-trim-right (remove-luks-suffix (car path)) char-set:digit)))
       (cons "disk"
             (if (string-suffix? "-part" trimmed)
               (cons (string-drop-right trimmed 5) path)
               path))))
    ; imsm = ("imsm" "imsm0_Volume1_0_part1")
    ((equal? "imsm" (car path))
     (format #t "REMOVE-PARTITION: ~S\n" path)
     (let*
       ((s (string-split (remove-luks-suffix (cadr path)) #\_))
        (last-path (car (reverse s))))
       (if (string-prefix? "part" last-path)
         (cons "disk" (cons (string-drop-right (remove-luks-suffix (cadr path)) (+ 1 (string-length last-path))) (cdr path)))
         (cons "imsm" (cons (car s) (cdr path))))))
    (#t
     (let* ((unluksed (remove-luks-suffix (car path)))
            (trimmed (string-trim-right unluksed char-set:digit)))
       (cons "disk"
             (if (equal? trimmed unluksed)
               path
               (cons trimmed path)))))))

(define (look-for-obj name)
  (let* ((devpath
           (let ((newpath (remove-evms-prefix name)))
             (cond
               ((equal? "btrfs" (car newpath))
                (let
                  ((normalized-path (normalize-btrfs-path newpath)))
                  (list (car normalized-path) (cadr normalized-path)
                        (string-join (cddr normalized-path) "|" 'infix))))
               ((member (car newpath) '("lvm2" "md"))
                newpath)
               (#t
                (remove-partition newpath)))))
         (subtree (member (vector
                            (cond
                              ((equal? "btrfs" (car devpath)) "BtrFS")
                              ((equal? "lvm2" (car devpath)) "LVM")
                              ((equal? "md" (car devpath)) "RAID")
                              ((equal? "imsm" (car devpath)) "IMSM")
                              (#t "Disks"))
                            "" "") (global 'tree)))
         (obj-vector
           (if (and (not (null? subtree)) (pair? devpath))
             (look-in-subtree (cadr subtree) (cdr devpath))
             #()))
         (fs-details
           (or (and (equal? (vector-ref obj-vector 2) "Unknown") "")
               (string-append
                 (cond
                   ((equal? "" (vector-ref obj-vector 2)) "")
                   ((string-null? (car (vector-ref obj-vector 2))) "")
                   (#t (string-append (_ "as ") (car (vector-ref obj-vector 2)))))
                 " "
                 (if (string-null? (vector-ref obj-vector 3))
                   ""
                   (string-append (_ "mounted at ") (vector-ref obj-vector 3)))))))
    (cond
      ((equal? "btrfs" (car devpath))
       (string-join `("BtrFS Subvolume" ,(caddr devpath)
                      "on" ,(cadr devpath)
                      ,fs-details)))
      ((equal? "lvm2" (car devpath))
       (string-join `("LVM VG" ,(cadr devpath)
                      "LV" ,(caddr devpath)
                      ,fs-details)))
      ((equal? "md" (car devpath))
       (string-join `("MD" ,(cadr devpath)
                      ,fs-details)))
      (#t
       (string-join `(,(_ "Partition")
                       ,(last devpath)
                       ,fs-details))))))

(define (look-in-subtree subtree path)
  (format #t "LOOK-IN-SUBTREE ~S ~S\n" subtree path)
  (if (or (null? subtree) (null? path))
    subtree
    (let ((res
            (member path subtree
                    (lambda (x y)
                      (and (vector? y)
                           (equal? (car x) (car (vector-ref y 0))))))))
      (if (null? (cdr path))
        (car res)
        (look-in-subtree
          (cadr res)
          (cdr path))))))

(define (operation-details type flag)
  (let ((pending (delete-duplicates (global 'operations))))
    (string-join
      (map
        (lambda (x)
	  (cond
	    ((equal? 'delete flag) (string-join `(,(_ "Cleaning volume")
						   ,(last (remove-evms-prefix (car x))))))
	    (#t
	     (look-for-obj (car x)))))
        (filter
          (lambda (x)
            (and (equal? (cadr x) type)
                 (member flag (caddr x))))
          pending))
      "<br>")))

(define (operation-list)
  (string-join
    (list
      (_ "Pending operations:")
      (operation-details 'volume 'delete)
      (operation-details 'volume 'activate)
      (operation-details 'subvolume 'activate))
    "<br><br>"))

(document:id changes-overview
             (gridbox width 480
                      (textbox (operation-list) alterability #f)
                      (label (_ "Apply requested changes?"))
                      (hbox
                        (button (_ "OK") (when clicked (document:end #t)))
                        (button (_ "Cancel") (when clicked (document:end #f))))))
