;  -*- scheme -*-

(document:surround "/std/frame")
(document:insert "/std/functions")

(document:insert "/vm/functions")

;----------------------------------------------------------------
(define (evms-open)
  (woo-catch/message
   (lambda()
     (woo-write "/evms/control" 'control 'open 'installer #t)
     (woo-write "/evms/control" 'control 'update))))

(define (current-profile)
  (string-append
   "/evms/profiles/"
   (list-ref (woo-list-names "/evms/profiles") (rl current))))

(define (profile-apply)
  (woo 'apply (current-profile)
       'commit (not (nocommit state)) 'clearall (clearall state) 'exclude (excluded-disks)))

(define (profile-commit . args)
  (evms-with-splash
   (lambda()
     (woo-catch/message
      (lambda ()
        (woo-write "/evms/control" 'control 'commit)
        (apply woo-write "/evms/control" 'control 'close args))))))

;---------------------------------------------------------------
(define (excluded-disks)
  (let ((rows (map cdr (cl checklist-rows)))
        (groups (caar (woo 'check (current-profile)))))
    (apply append (filter-map (lambda (name state) (and (not state) name)) groups rows))))

(define (make-group-row disks)
  (string-append " " (string-join disks ", ")))

(define (make-disk-row disk)
  (let ((vals (woo-leaf-props (string-append "/evms/storage/disks/" disk) '(id size free))))
    (string-append " " disk " [" (car vals) "]  " (caddr vals) (_ " of ") (cadr vals) (_ " free"))))

(define (make-group-rows)
  (let ((groups (caar (woo 'check (current-profile)))))
      (append-map
	(lambda (x)
	  (list (make-group-row x)))
	groups)))

(define (make-disk-rows)
  (map make-disk-row (woo-list-names "/evms/storage/disks")))

(define (make-action-rows)
  (map
   (lambda (p)
     (let ((path (string-append "/evms/profiles/" p)))
       (let ((reqsize (caar (woo 'size path)))
             (title (_ (caar (woo 'title path)))))
         (if (string-null? reqsize) title
             (string-append title " [ " (_ "free space required:") reqsize " ]")))))
   (woo-list-names "/evms/profiles")))

(define (update-controls)
  (frame:next-activity
    (and (not (= -1 (rl current)))
         (any cdr (cl checklist-rows)))))

(define (warn-on-clearall)
  (eqv? 'ok (document:popup-warning
             (string-append
              "<center><p><b>"
              (_ "All data from selected groups will be lost, you won't be able to recover it! Continue?")
              "</b></p></center>")
             'cancel 'ok)))

;---------------------------------------------------------------
(document:id
 disktab
 (gridbox columns "50;50"
  (groupbox
   (_ "Select disk groups for use")
   (document:id cl (checklist checklist-rows '()
                              (when clicked (update-controls)))))
  (groupbox
   (_ "Available disks")
   (document:id dl (listbox rows '())))))

(groupbox
 (_ "Select profile")
 (document:id
  rl
  (radiolistbox rows '()
		(when clicked (update-controls)))))

(groupbox
 (_ "Options")
 (document:id
  clearall
  (checkbox (_ "Clear selected before applying profile") state #f
            (when toggled
              (or (and (clearall state)
                       (warn-on-clearall))
                  (clearall state #f)))))
 (document:id nocommit (checkbox (_ "Let me make my changes after profile applying") state #t)))

(frame:on-next
 (lambda ()
   (let ((reply (profile-apply)))
     (format #t "<* ~S\n" reply)
     (cond
      ((eqv? (caar reply) 'nexthop)
       (frame:replace (cadar reply) 'installer #t 'nexthop '/vm/ortodox)
       'cancel)
      ((eqv? (caar reply) #t)
       (let loop ((ret (profile-commit)))
         (format #t "RET: ~S\n" ret)
         (cond ((null? ret)             ; all went good
                #t)
               ((not ret)               ; fail
                (woo-write "/evms/control" 'control 'reset)
                'cancel)
               (#t                      ; show warn
                (case (document:popup-warning (cadar ret) 'cancel 'ok)
                  ((ok)                 ; ignore this warning
                   (loop (profile-commit 'ignore (caar ret))))
                  ((cancel)             ; let user fix things
                   (frame:replace '/vm/table 'installer #t)
                   'cancel))))))
      (#t
       (woo-write "/evms/control" 'control 'reset)
       (woo-write "/evms/control" 'control 'update)
       (document:popup-critical
        (apply string-append
               "<center><p><b>"
               (_ "Unable to apply selected profile, not enough disk space")
               "</b></p></center>"
               (map cadr reply)) 'ok)
       'cancel)))))

(document:root
 (when loaded
   (and (evms-open)
        (if (or (eqv? (global 'frame:direction) 'next)
                (global 'lemmein))
            (evms-with-splash
             (lambda()
               (rl rows (make-action-rows))
               (rl current 0)
               (dl rows (make-disk-rows))
               (cl checklist-rows (make-group-rows))
               (frame:back-activity #t)))
            (frame:replace '/vm/table 'installer #t)))))
