;  -*- scheme -*-

(document:surround "/std/frame")
(document:insert "/std/functions")

(document:insert "/vm/functions")

(document:envelop with-attributes (expanded))

;----------------------------------------------------------------
(define nexthop (global 'nexthop))
(define installer (global 'installer))
  
(define (woo-tree url)
  (and-let* ((tree (woo-catch
                    (lambda () (woo-try 'tree url))
                    (lambda x #f))))
            (caddr (woo-first-command tree))))

(define (woo-task url op)
  (woo-catch
   (lambda () (woo-try 'task url 'operation op))
   (lambda x #f)))

(define (evms-close . args)
  (woo-catch
   (lambda () (apply woo-write "/evms/control" 'control 'close args))
   (lambda(msg) (document:popup-critical (_ msg) 'ok) #f)))

(define (evms-write-control control)
  (evms-with-splash
   (lambda ()
     (and (woo-catch
           (lambda () (woo-write "/evms/control" 'control control))
           (lambda(msg) (document:popup-critical (_ msg) 'ok) #f))
          (evms-update)))))

(define (evms-read-control control)
  (woo-get-option
   (woo-first-command
    (woo-read "/evms/control" 'control control))
   control))

(define (evms-action-title action)
  (assv-ref action-alist action))

(define (evms-with-confirmation func message . rest)
  (if message
      (if (eqv? 'ok (document:popup-question (_ message) 'ok 'cancel))
          (apply func rest)
          '((cancel)))
      (apply func rest)))

(define (evms-popup action next-action path)
  (document:popup '/vm/popup
                  'task-path path
                  'title (evms-action-title action)
                  'next-title (evms-action-title next-action)))

(define (evms-choice action next-action . paths)
  (document:popup '/vm/choice
                  'action action        ; passthru
                  'next-action next-action
                  'task-paths paths
                  'title (evms-action-title action)))

(define (evms-action action)
  (let loop ((reply (woo-task (storage-path) action)))
    (format #t "<* ~S\n" reply)
    (if (list? reply)
        (cond
         ((eqv? (caar reply) 'task)
          (loop (apply evms-with-confirmation evms-popup (cdar reply))))
         ((eqv? (caar reply) 'choice)
          (loop (apply evms-with-confirmation evms-choice (cdar reply))))
         ((eqv? (caar reply) 'cancel)   ; cancelled by user
          (format #t "CANCELLED\n")
          (evms-update))
         ((eqv? (caar reply) #t)        ; finished ok
          (evms-update))
         (#t (document:popup-critical
              (_ (apply string-append (map cadr reply))) 'ok)))
        (document:popup-information
         (string-append
          "<center><p><b>"
          (_ "Congrats! You've found a secret level!")
          "</b></p></center>")
         'ok))))

(define (evms-update)
  (woo-write "/evms/control" 'control 'update)
  (update-data)
  (update-controls)
  (update-action-buttons))

(define (update-data)
    (tr expanded #t tree-items (woo-tree "/evms/tree")))

(define (update-controls)
  (or installer
      (let ((pending (not (not (evms-read-control 'pending)))))
        (apply-button activity pending)
        (reset-button activity pending))))

(define (evms-on-next)
  (or (and (evms-assigned)
           (evms-efi)
           (evms-ptable)
           (evms-sufficient-space)
           (evms-applied)
           (evms-closed))
      'cancel))

(define (evms-on-back)
  (tr current '(0))
  (or (evms-applied)
      (evms-write-control 'reset))
  (frame:replace nexthop 'installer installer 'lemmein #t)
  'cancel)

(define (evms-assigned)
  (cond ((member "/" (evms-read-control 'assigned)) #t)
        (#t (document:popup-warning (_  "Please assign \"/\" mount point") 'ok) #f)))

(define (evms-sufficient-space)
  (cond ((evms-read-control 'sufficient) #t)
        ((evms-read-control 'sufficient-critical)
         (eqv? 'ok (document:popup-warning
           (string-append
             "<p>"
             (_  "\"/\" volume is large enough for installation but you may be unable to update or use the system.")
             "</p>"
             "<p>"
             (_  "Recommended size for \"/\" is at least ")
             (evms-read-control 'required-size)
             "</p>"
             "<p>"
             (_  "Continue?")
             "</p>")
           'ok
           'cancel)))
        (#t (document:popup-warning
              (string-append
                (_  "Insufficient disk space for installation. \"/\" should be at least ")
                (evms-read-control 'required-size))) #f)))

(define (evms-efi)
  (cond ((not (evms-read-control 'efi)) #t)
        ((evms-read-control 'efi-on-sb09) (document:popup-warning (_  "/boot/efi on superblock 0.90 or 1.0 is supported for removable devices only") 'ok) #t)
        ((evms-read-control 'efi-on-raid) (document:popup-warning (_  "/boot/efi on superblock > 1.0 is not supported") 'ok) #f)
        ((member "/boot/efi" (evms-read-control 'assigned)) #t)
        (#t (document:popup-warning (_  "To install grub-efi, ESP partition (FAT32, minimum 256 MiB, mountpoint /boot/efi) is required") 'ok) #f)))

(define (evms-ptable)
  (cond ((evms-read-control 'efi) #t)
        ((evms-read-control 'any-ptable) #t)
        (#t (document:popup-warning (_  "GRUB could not be installed; please create either partition table or plain volume") 'ok) #f)))

(define (get-first-duplicate-item lst)
  (cond ((null? lst) '())
        ((member (car lst) (cdr lst)) (car lst))
        (else (get-first-duplicate-item (cdr lst)))))

(define (evms-find-duplicate-mntpoint)
  (get-first-duplicate-item (evms-read-control 'assigned)))

(define (evms-applied)
  (let ((postponed (evms-read-control 'postponed))
        (applied (not (evms-read-control 'pending)))
        (duplicate-mntpoint (evms-find-duplicate-mntpoint)))
    (cond ((and applied (null? postponed) (null? duplicate-mntpoint)) #t)
          ((not (null? duplicate-mntpoint))
           (document:popup-critical
            (string-append "<center><p><b>"
                           (_ "Multiple filesystems are assigned the same mountpoint (")
                           duplicate-mntpoint
                           "). "
                           (_ "Please fix this by changing duplicated mountpoints.")
                           "</b></p></center>") 'ok)
           #f)
          ((null? postponed)
           (and (document:popup
                  '/vm/details
                  'tree (woo-tree "/evms/tree")
                  'operations (evms-read-control 'pending))
                (evms-write-control 'commit)))
          (#t (and (eqv? 'ok (document:popup-warning
                              (string-append
                               "<center><p><b>"
                               (_  "All data on following partitions will be lost:")
                               "</b></p></center><center><b>"
                               (string-join postponed "</b></center><center><b>" 'infix)
                               "</b></center><center><p><b>" (_ "Continue?") "</b></p></center>")
                              'ok 'cancel))
                   (evms-write-control 'commit))))))

(define (evms-closed)
  (let loop ((ret (evms-close)))
    (format #t "CLOSE:: ~S\n" ret)
    (cond ((null? ret) #t)              ; all went good
          ((not ret) #f)                ; fail, stay here
          (#t                           ; show warn
           (case (document:popup-warning (cadar ret) 'cancel 'ok)
             ((ok)                     ; ignore this warning
              (loop (evms-close 'ignore (caar ret))))
             ((cancel)                 ; let user fix things
              #f))))))

;---------------------------------------------------------------
(define (make-action-buttons)
  (let ((make-button
         (lambda (act)
           (button (cdr act) activity #f visibility #f
                   (when clicked (evms-action (car act)))))))
    (let ((buttons (map make-button action-alist)))
      (cell-set! action-buttons buttons)
      buttons)))

(define (update-action-buttons)
  (let ((path (storage-path)))
    (let ((ops (if path (woo-leaf-ops path) '())))
      (map
       (lambda (w descr)
         (let ((act (assv (car descr) ops)))
           (if (pair? act) ;; action defined, at least
               (if (cdr act) ;; and allowed
                   (w visibility #t activity #t)
                   ;; (w visibility #t activity #f)
                   (w visibility #f activity #f))
               (w visibility #f activity #f))))
       (cell-ref action-buttons)
       action-alist))))

;---------------------------------------------------------------
(define action-buttons (make-cell '()))

(define (storage-path)
  (string-join (cons "/evms/tree" (tr tree-text)) "/" 'infix))

(define storage-rows
  `(
    ,(_ "   Name   ")
    ,(_ "Size [ free ]")
    ,(_ "Filesystem")
    ,(if installer
         (_ "Mountpoint")
         (_ "Mounted on"))
    ,(_ "Mount options")))

(define storage-action-alist
  `((op-raid-create . ,(_ "Create RAID"))
    (op-raid-destroy . ,(_ "Destroy"))

    (op-vg-create . ,(_ "Create Volume Group"))
    (op-vg-expand . ,(_ "Expand"))
    (op-vg-shrink . ,(_ "Shrink"))
    (op-vg-destroy . ,(_ "Destroy"))

    (op-lv-create . ,(_ "Create Volume"))
    (op-luks-lv . ,(_ "Create Encrypted Volume"))
    (op-lv-expand . ,(_ "Expand"))
    (op-lv-shrink . ,(_ "Shrink"))
    (op-lv-destroy . ,(_ "Destroy"))

    (op-pt-create . ,(_ "Create Partition"))
    (op-pt-destroy . ,(_ "Destroy Partition Table"))
    (op-segment-create . ,(_ "Create Partition"))
    (op-luks-segment . ,(_ "Create Encrypted Partition"))
    (op-luks-pt . ,(_ "Create Encrypted Partition"))
    (op-segment-expand . ,(_ "Expand"))
    (op-segment-shrink . ,(_ "Shrink"))
    (op-segment-destroy . ,(_ "Destroy"))

    (op-volume-create . ,(_ "Create Volume"))
    (op-volume-expand . ,(_ "Expand"))
    (op-volume-shrink . ,(_ "Shrink"))
    (op-volume-destroy . ,(_ "Destroy"))

    (op-luks-create . ,(_ "Create Encrypted Volume"))
    (op-luks-destroy . ,(_ "Destroy"))))

(define volume-action-alist
  (let ((install  `((op-volume-mkfs . ,(_ "Make Filesystem"))
                    (op-volume-unmkfs . ,(_ "Remove Filesystem"))
                    (op-volume-setmntent . ,(_ "Edit Mountpoint"))
                    (op-subvol-create . ,(_ "Create Subvolume"))
                    (op-subvol-destroy . ,(_ "Destroy Subvolume"))
                    (op-subvol-setmntent . ,(_ "Edit Subvolume Mountpoint"))))
        (regular `((op-volume-mkfs . ,(_ "Make Filesystem"))
                   (op-volume-fsck . ,(_ "Check Filesystem"))
                   (op-volume-mount . ,(_ "Mount Filesystem"))
                   (op-volume-umount . ,(_ "Umount Filesystem"))
                   (op-volume-unmkfs . ,(_ "Remove Filesystem")))))
    (if installer install regular)))

(define action-alist (append storage-action-alist volume-action-alist))

;---------------------------------------------------------------
(groupbox
 ""
 (document:id
  tr
  (tree columns (length storage-rows)
        header (apply vector storage-rows)
        expanded #t
        tree-items '()
        (when selected (update-action-buttons))))
 (apply box
        orientation 'horizontal
        (make-action-buttons)))

(or installer
    (box
     orientation 'horizontal
     (document:id
      reset-button
      (button (_ "Reset") activity #f
              (when clicked (evms-write-control 'reset))))
     
     (document:id
      apply-button
      (button (_ "Apply") activity #f
              (when clicked (evms-write-control 'commit))))))

(frame:on-next evms-on-next)
(frame:on-back evms-on-back)

(document:root (when loaded (evms-update)))
(document:root (when destroyed (evms-close)))
